<?php

/**
 * This is a very simple API that implements a method to update
 * the Yavli Ad Server code.
 *
 * You may adapt this class in ways to better suit your website.
 * For example, you might want to add support for storing the
 * Yavli Ad Server code in your local database rather than
 * storing it in a file.
 */
class YResilience_API {

  private $data = null;

  private $has_curl = false;
  private $has_fopen = false;

  private $website_id = null;
  private $api_key = null;
  private $api_url = "api.yavli.com/v2";
  private $cache_file = null;

  /**
   * Set up variables and detect available HTTPS methods.
   */
  public function __construct($cache_file = "data.json") {
    $this->cache_file = $cache_file;

    $this->api_result = null;
    $this->api_status = null;

    $this->api_reachable = null;

    $this->api_error_code = null;
    $this->api_error_detail = null;

    if (ini_get('allow_url_fopen')) {
      $this->has_fopen = true;
    }
    if (function_exists('curl_version')) {
      $this->has_curl = true;
    }
  }

  public function set_credentials($website_id, $api_key) {
    $this->website_id = $website_id;
    $this->api_key = $api_key;
  }

  /**
   * Retrieve the latest version of the Yavli Ad Server Code
   * from the Yavli API.
   */
  public function refresh_ad_server_code() {
    $this->api_request("/get-ad-server");
    if ($this->api_reachable == true && $this->api_status == 200) {
      return true;
    }
    return false;
  }

  /**
   * Fetch the locally stored Yavli Ad Server Code.
   */
  public function get_ad_server_code() {
    if ($this->data == null) {
      $this->load_data();
    }
    if ($this->data != null) {
      return $this->data->data->init_js;
    }
    throw new Exception("No data found. You must run the API cron job before you can do this.");
  }

  /**
   * Save Ad Server Code locally. This method can be replaced with one that
   * stores the Ad Server code in a local database instead of a simple file.
   */
  public function save_data() {
    if ($this->api_reachable == true && $this->api_status == 200) {
      try {
        file_put_contents($this->cache_file, json_encode($this->api_result));
      } catch (Exception $e) {
        throw new Exception("Cache file is not writable: " . $this->cache_file);
      }
      $this->data = $this->api_result;
    }
  }

  public function get_error_msg() {
    $msg = "API Reachable: ";
    $msg .= ($this->api_reachable ? "Yes" : "No");
    $msg .= ", API Status: '".$this->api_status."', ";
    $msg .= "API Error Code: '".$this->api_error_code."', ";
    $msg .= "API Error Detail: '".$this->api_error_detail."'";
    return $msg;
  }

  private function get_api_url($path) {
    return "http://" . $this->api_url . $path;
  }

  private function load_data() {
    if (file_exists($this->cache_file)) {
      $this->data = json_decode(file_get_contents($this->cache_file));
    }
  }

  private function get_auth_headers() {
    if ($this->website_id == null || $this->api_key == null) {
      throw new Exception("Credentials must be set in order to make API calls");
    }
    return array("X-WebsiteID" => $this->website_id, "X-ApiKey" => $this->api_key);
  }

  protected function api_request($action) {
    $url = $this->get_api_url($action);

    $hvars = $this->get_auth_headers();
    if ($this->has_fopen) {
      $f_headers = "";
      foreach ($hvars as $hkey => $hval) {
        $f_headers .= $hkey.": ".$hval."\r\n";
      }
      $context = stream_context_create(array(
        'http'=>array(
          'header'=>$f_headers,
          'method'=>"GET"
        )
      ));
      $response = file_get_contents($url, false, $context);
    } else
    if ($this->has_curl) {
      $curl_headers = array();
      foreach ($hvars as $hkey => $hval) {
        $curl_headers[] = $hkey.": ".$hval;
      }
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_HTTPHEADER, $curl_headers);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
      $response = curl_exec($ch);
      curl_close($ch);
    } else {
      throw new Exception("No HTTPS capable method detected, need Curl or PHP with allow_url_include = On.");
    }

    $result = json_decode($response);
    if ($result != null) {
      $this->api_reachable = true;

      if (property_exists($result, "error")) {
        $this->api_status = $result->error->status;
        $this->api_error_code = $result->error->code;
        $this->api_error_detail = $result->error->detail;
      } else {
        $this->api_status = 200;
        $this->api_result = $result;
      }

    } else {
      $this->api_reachable = false;
    }
  }
}
?>
